@@ -42,6 +42,8 @@ from ludwig.constants import (
     BATCH_SIZE,
     EVAL_BATCH_SIZE,
     FULL,
+    HYPEROPT,
+    HYPEROPT_WARNING,
     LEARNING_RATE,
     PREPROCESSING,
     TEST,
@@ -61,7 +63,6 @@ from ludwig.globals import (
     set_disable_progressbar,
     TRAIN_SET_METADATA_FILE_NAME,
 )
-from ludwig.marshmallow.marshmallow_schema_utils import load_config_with_kwargs
 from ludwig.models.ecd import ECD
 from ludwig.models.inference import InferenceModule
 from ludwig.models.predictor import (
@@ -72,6 +73,8 @@ from ludwig.models.predictor import (
 )
 from ludwig.models.trainer import Trainer
 from ludwig.modules.metric_modules import get_best_function
+from ludwig.schema import validate_config
+from ludwig.schema.utils import load_config_with_kwargs
 from ludwig.utils import metric_utils
 from ludwig.utils.data_utils import (
     figure_data_format,
@@ -85,7 +88,6 @@ from ludwig.utils.defaults import default_random_seed, merge_with_defaults
 from ludwig.utils.fs_utils import makedirs, open_file, path_exists, upload_output_directory
 from ludwig.utils.misc_utils import get_file_names, get_output_directory
 from ludwig.utils.print_utils import print_boxed
-from ludwig.utils.schema import validate_config
 
 logger = logging.getLogger(__name__)
 
@@ -336,6 +338,10 @@ class LudwigModel:
             `(training_set, validation_set, test_set)`.
             `output_directory` filepath to where training results are stored.
         """
+        if HYPEROPT in self.config:
+            print_boxed("WARNING")
+            logger.info(HYPEROPT_WARNING)
+
         # setup directories and file names
         if model_resume_path is not None:
             if path_exists(model_resume_path):
@@ -1072,6 +1078,10 @@ class LudwigModel:
             `(training_set, validation_set, test_set)`, `output_directory`
             filepath string to where results are stored.
         """
+        if HYPEROPT in self.config:
+            print_boxed("WARNING")
+            logger.info(HYPEROPT_WARNING)
+
         (train_stats, preprocessed_data, output_directory) = self.train(
             dataset=dataset,
             training_set=training_set,
@@ -1442,7 +1452,7 @@ class LudwigModel:
         model_hyperparameters_path = os.path.join(save_path, MODEL_HYPERPARAMETERS_FILE_NAME)
         save_json(model_hyperparameters_path, self.config)
 
-    def to_torchscript(self):
+    def to_torchscript(self, model_only: bool = False):
         """Converts the trained LudwigModule, including preprocessing and postprocessing, to Torchscript.
 
         The scripted module takes in a `Dict[str, Union[List[str], Tensor]]` as input.
@@ -1455,14 +1465,21 @@ class LudwigModel:
         Similarly, the output will be a dictionary of dictionaries, where each feature has its own dictionary of
         outputs. The outputs will be a list of strings for predictions with string types, while other outputs will be
         tensors of varying dimensions for probabilities, logits, etc.
+
+        Args:
+            model_only (bool, optional): If True, only the ECD model will be converted to Torchscript. Else,
+                preprocessing and postprocessing will also be converted to Torchscript.
         """
         self._check_initialization()
-        inference_module = InferenceModule(self.model, self.config, self.training_set_metadata)
-        return torch.jit.script(inference_module)
+        if model_only:
+            return self.model.to_torchscript()
+        else:
+            inference_module = InferenceModule(self.model, self.config, self.training_set_metadata)
+            return torch.jit.script(inference_module)
 
-    def save_torchscript(self, save_path: str):
+    def save_torchscript(self, save_path: str, model_only: bool = False):
         """Saves the Torchscript model to disk."""
-        inference_module = self.to_torchscript()
+        inference_module = self.to_torchscript(model_only=model_only)
         inference_module.save(os.path.join(save_path, INFERENCE_MODULE_FILE_NAME))
 
     def _check_initialization(self):
