@@ -123,6 +123,8 @@ EXECUTOR = "executor"
 MINIMIZE = "minimize"
 MAXIMIZE = "maximize"
 SAMPLER = "sampler"
+SEARCH_ALG = "search_alg"
+SCHEDULER = "scheduler"
 PARAMETERS = "parameters"
 
 NAME = "name"
@@ -163,7 +165,9 @@ AUTOML_DEFAULT_IMAGE_ENCODER = "stacked_cnn"
 HYPEROPT_WARNING = (
     "You are running the ludwig train command but there’s a hyperopt section present in your config. "
     "It will be ignored. If you want to run hyperopt you should use the following command: ludwig "
-    "hyperopt\n\n Do you want to continue? "
+    "hyperopt\n\n"
 )
 
+CONTINUE_PROMPT = "Do you want to continue? "
+
 DEFAULT_AUDIO_TENSOR_LENGTH = 70000
