@@ -1,5 +1,4 @@
 #! /usr/bin/env python
-# coding=utf-8
 # Copyright (c) 2019 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -24,16 +23,32 @@ import uuid
 from typing import List
 
 import numpy as np
+import torch
+import torchaudio
 import yaml
 
-from ludwig.constants import (AUDIO, BAG, BINARY, CATEGORY, DATE, H3, IMAGE,
-                              NUMERICAL, PREPROCESSING, SEQUENCE, SET,
-                              TEXT, TIMESERIES, TYPE, VECTOR, NAME)
-from ludwig.contrib import contrib_command, contrib_import
+from ludwig.constants import (
+    AUDIO,
+    BAG,
+    BINARY,
+    CATEGORY,
+    DATE,
+    H3,
+    IMAGE,
+    NAME,
+    NUMBER,
+    PREPROCESSING,
+    SEQUENCE,
+    SET,
+    TEXT,
+    TIMESERIES,
+    TYPE,
+    VECTOR,
+)
+from ludwig.contrib import add_contrib_callback_args
 from ludwig.globals import LUDWIG_VERSION
 from ludwig.utils.data_utils import save_csv
 from ludwig.utils.h3_util import components_to_h3
-from ludwig.utils.horovod_utils import is_on_master
 from ludwig.utils.misc_utils import get_from_registry
 from ludwig.utils.print_utils import print_ludwig
 
@@ -42,38 +57,38 @@ logger = logging.getLogger(__name__)
 letters = string.ascii_letters
 
 DATETIME_FORMATS = {
-    '%m-%d-%Y': '{m:02d}-{d:02d}-{Y:04d}',
-    '%m-%d-%Y %H:%M:%S': '{m:02d}-{d:02d}-{Y:04d} {H:02d}:{M:02d}:{S:02d}',
-    '%m/%d/%Y': '{m:02d}/{d:02d}/{Y:04d}',
-    '%m/%d/%Y %H:%M:%S': '{m:02d}/{d:02d}/{Y:04d} {H:02d}:{M:02d}:{S:02d}',
-    '%m-%d-%y': '{m:02d}-{d:02d}-{y:02d}',
-    '%m-%d-%y %H:%M:%S': '{m:02d}-{d:02d}-{y:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%m/%d/%y': '{m:02d}/{d:02d}/{y:02d}',
-    '%m/%d/%y %H:%M:%S': '{m:02d}/{d:02d}/{y:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%d-%m-%Y': '{d:02d}-{m:02d}-{Y:04d}',
-    '%d-%m-%Y %H:%M:%S': '{d:02d}-{m:02d}-{Y:04d} {H:02d}:{M:02d}:{S:02d}',
-    '%d/%m/%Y': '{d:02d}/{m:02d}/{Y:04d}',
-    '%d/%m/%Y %H:%M:%S': '{d:02d}/{m:02d}/{Y:04d} {H:02d}:{M:02d}:{S:02d}',
-    '%d-%m-%y': '{d:02d}-{m:02d}-{y:02d}',
-    '%d-%m-%y %H:%M:%S': '{d:02d}-{m:02d}-{y:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%d/%m/%y': '{d:02d}/{m:02d}/{y:02d}',
-    '%d/%m/%y %H:%M:%S': '{d:02d}/{m:02d}/{y:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%y-%m-%d': '{y:02d}-{m:02d}-{d:02d}',
-    '%y-%m-%d %H:%M:%S': '{y:02d}-{m:02d}-{d:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%y/%m/%d': '{y:02d}/{m:02d}/{d:02d}',
-    '%y/%m/%d %H:%M:%S': '{y:02d}/{m:02d}/{d:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%Y-%m-%d': '{Y:04d}-{m:02d}-{d:02d}',
-    '%Y-%m-%d %H:%M:%S': '{Y:04d}-{m:02d}-{d:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%Y/%m/%d': '{Y:04d}/{m:02d}/{d:02d}',
-    '%Y/%m/%d %H:%M:%S': '{Y:04d}/{m:02d}/{d:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%y-%d-%m': '{y:02d}-{d:02d}-{m:02d}',
-    '%y-%d-%m %H:%M:%S': '{y:02d}-{d:02d}-{m:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%y/%d/%m': '{y:02d}/{d:02d}/{m:02d}',
-    '%y/%d/%m %H:%M:%S': '{y:02d}/{d:02d}/{m:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%Y-%d-%m': '{Y:04d}-{d:02d}-{m:02d}',
-    '%Y-%d-%m %H:%M:%S': '{Y:04d}-{d:02d}-{m:02d} {H:02d}:{M:02d}:{S:02d}',
-    '%Y/%d/%m': '{Y:04d}/{d:02d}/{m:02d}',
-    '%Y/%d/%m %H:%M:%S': '{Y:04d}/{d:02d}/{m:02d} {H:02d}:{M:02d}:{S:02d}'
+    "%m-%d-%Y": "{m:02d}-{d:02d}-{Y:04d}",
+    "%m-%d-%Y %H:%M:%S": "{m:02d}-{d:02d}-{Y:04d} {H:02d}:{M:02d}:{S:02d}",
+    "%m/%d/%Y": "{m:02d}/{d:02d}/{Y:04d}",
+    "%m/%d/%Y %H:%M:%S": "{m:02d}/{d:02d}/{Y:04d} {H:02d}:{M:02d}:{S:02d}",
+    "%m-%d-%y": "{m:02d}-{d:02d}-{y:02d}",
+    "%m-%d-%y %H:%M:%S": "{m:02d}-{d:02d}-{y:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%m/%d/%y": "{m:02d}/{d:02d}/{y:02d}",
+    "%m/%d/%y %H:%M:%S": "{m:02d}/{d:02d}/{y:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%d-%m-%Y": "{d:02d}-{m:02d}-{Y:04d}",
+    "%d-%m-%Y %H:%M:%S": "{d:02d}-{m:02d}-{Y:04d} {H:02d}:{M:02d}:{S:02d}",
+    "%d/%m/%Y": "{d:02d}/{m:02d}/{Y:04d}",
+    "%d/%m/%Y %H:%M:%S": "{d:02d}/{m:02d}/{Y:04d} {H:02d}:{M:02d}:{S:02d}",
+    "%d-%m-%y": "{d:02d}-{m:02d}-{y:02d}",
+    "%d-%m-%y %H:%M:%S": "{d:02d}-{m:02d}-{y:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%d/%m/%y": "{d:02d}/{m:02d}/{y:02d}",
+    "%d/%m/%y %H:%M:%S": "{d:02d}/{m:02d}/{y:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%y-%m-%d": "{y:02d}-{m:02d}-{d:02d}",
+    "%y-%m-%d %H:%M:%S": "{y:02d}-{m:02d}-{d:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%y/%m/%d": "{y:02d}/{m:02d}/{d:02d}",
+    "%y/%m/%d %H:%M:%S": "{y:02d}/{m:02d}/{d:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%Y-%m-%d": "{Y:04d}-{m:02d}-{d:02d}",
+    "%Y-%m-%d %H:%M:%S": "{Y:04d}-{m:02d}-{d:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%Y/%m/%d": "{Y:04d}/{m:02d}/{d:02d}",
+    "%Y/%m/%d %H:%M:%S": "{Y:04d}/{m:02d}/{d:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%y-%d-%m": "{y:02d}-{d:02d}-{m:02d}",
+    "%y-%d-%m %H:%M:%S": "{y:02d}-{d:02d}-{m:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%y/%d/%m": "{y:02d}/{d:02d}/{m:02d}",
+    "%y/%d/%m %H:%M:%S": "{y:02d}/{d:02d}/{m:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%Y-%d-%m": "{Y:04d}-{d:02d}-{m:02d}",
+    "%Y-%d-%m %H:%M:%S": "{Y:04d}-{d:02d}-{m:02d} {H:02d}:{M:02d}:{S:02d}",
+    "%Y/%d/%m": "{Y:04d}/{d:02d}/{m:02d}",
+    "%Y/%d/%m %H:%M:%S": "{Y:04d}/{d:02d}/{m:02d} {H:02d}:{M:02d}:{S:02d}",
 }
 
 
@@ -81,7 +96,7 @@ def generate_string(length):
     sequence = []
     for _ in range(length):
         sequence.append(random.choice(letters))
-    return ''.join(sequence)
+    return "".join(sequence)
 
 
 def build_vocab(size):
@@ -96,40 +111,36 @@ def return_none(feature):
 
 
 def assign_vocab(feature):
-    feature['idx2str'] = build_vocab(feature.get('vocab_size', 10))
+    feature["idx2str"] = build_vocab(feature.get("vocab_size", 10))
 
 
 def build_feature_parameters(features):
     feature_parameters = {}
     for feature in features:
-        fearure_builder_function = get_from_registry(
-            feature[TYPE],
-            parameters_builders_registry
-        )
-
-        feature_parameters[feature[NAME]] = fearure_builder_function(feature)
+        feature_builder_function = get_from_registry(feature[TYPE], parameters_builders_registry)
+        feature_parameters[feature[NAME]] = feature_builder_function(feature)
     return feature_parameters
 
 
 parameters_builders_registry = {
-    'category': assign_vocab,
-    'text': assign_vocab,
-    'numerical': return_none,
-    'binary': return_none,
-    'set': assign_vocab,
-    'bag': assign_vocab,
-    'sequence': assign_vocab,
-    'timeseries': return_none,
-    'image': return_none,
-    'audio': return_none,
-    'date': return_none,
-    'h3': return_none,
-    VECTOR: return_none
+    "category": assign_vocab,
+    "text": assign_vocab,
+    "number": return_none,
+    "binary": return_none,
+    "set": assign_vocab,
+    "bag": assign_vocab,
+    "sequence": assign_vocab,
+    "timeseries": return_none,
+    "image": return_none,
+    "audio": return_none,
+    "date": return_none,
+    "h3": return_none,
+    VECTOR: return_none,
 }
 
 
 def build_synthetic_dataset(dataset_size: int, features: List[dict]):
-    """Symthesizes a dataset for testing purposes
+    """Symthesizes a dataset for testing purposes.
 
     :param dataset_size: (int) size of the dataset
     :param features: (List[dict]) list of features to generate in YAML format.
@@ -144,8 +155,8 @@ def build_synthetic_dataset(dataset_size: int, features: List[dict]):
         {name: text_2, type: text, vocab_size: 20, max_len: 20},
         {name: category_1, type: category, vocab_size: 10},
         {name: category_2, type: category, vocab_size: 15},
-        {name: numerical_1, type: numerical},
-        {name: numerical_2, type: numerical},
+        {name: number_1, type: number},
+        {name: number_2, type: number},
         {name: binary_1, type: binary},
         {name: binary_2, type: binary},
         {name: set_1, type: set, vocab_size: 20, max_len: 20},
@@ -177,147 +188,125 @@ def build_synthetic_dataset(dataset_size: int, features: List[dict]):
 def generate_datapoint(features):
     datapoint = []
     for feature in features:
-        if ('cycle' in feature and feature['cycle'] is True and
-                feature[TYPE] in cyclers_registry):
+        if "cycle" in feature and feature["cycle"] is True and feature[TYPE] in cyclers_registry:
             cycler_function = cyclers_registry[feature[TYPE]]
             feature_value = cycler_function(feature)
         else:
-            generator_function = get_from_registry(
-                feature[TYPE],
-                generators_registry
-            )
+            generator_function = get_from_registry(feature[TYPE], generators_registry)
             feature_value = generator_function(feature)
         datapoint.append(feature_value)
     return datapoint
 
 
 def generate_category(feature):
-    return random.choice(feature['idx2str'])
+    return random.choice(feature["idx2str"])
 
 
-def generate_numerical(feature):
-    return random.uniform(
-        feature['min'] if 'min' in feature else 0,
-        feature['max'] if 'max' in feature else 1
-    )
+def generate_number(feature):
+    return random.uniform(feature["min"] if "min" in feature else 0, feature["max"] if "max" in feature else 1)
 
 
 def generate_binary(feature):
-    p = feature['prob'] if 'prob' in feature else 0.5
-    return np.random.choice([True, False], p=[p, 1 - p])
+    choices = feature.get("bool2str", [False, True])
+    p = feature["prob"] if "prob" in feature else 0.5
+    return np.random.choice(choices, p=[1 - p, p])
 
 
 def generate_sequence(feature):
-    length = feature.get('max_len', 10)
-    if 'min_len' in feature:
-        length = random.randint(feature['min_len'], length)
-    sequence = [random.choice(feature['idx2str']) for _ in range(length)]
-    return ' '.join(sequence)
+    length = feature.get("max_len", 10)
+    if "min_len" in feature:
+        length = random.randint(feature["min_len"], length)
+    sequence = [random.choice(feature["idx2str"]) for _ in range(length)]
+    if "vocab_size" not in feature:
+        feature["vocab_size"] = len(feature["idx2str"])
+    feature["vocab_size"] = feature["vocab_size"] + 4  # For special symbols: START, STOP, PAD, UNK.
+    return " ".join(sequence)
 
 
 def generate_set(feature):
     elems = []
-    for _ in range(random.randint(0, feature.get('max_len', 3))):
-        elems.append(random.choice(feature['idx2str']))
-    return ' '.join(list(set(elems)))
+    for _ in range(random.randint(0, feature.get("max_len", 3))):
+        elems.append(random.choice(feature["idx2str"]))
+    return " ".join(list(set(elems)))
 
 
 def generate_bag(feature):
     elems = []
-    for _ in range(random.randint(0, feature.get('max_len', 3))):
-        elems.append(random.choice(feature['idx2str']))
-    return ' '.join(elems)
+    for _ in range(random.randint(0, feature.get("max_len", 3))):
+        elems.append(random.choice(feature["idx2str"]))
+    return " ".join(elems)
 
 
 def generate_text(feature):
-    length = feature.get('max_len', 10)
+    length = feature.get("max_len", 10)
     text = []
     for _ in range(random.randint(length - int(length * 0.2), length)):
-        text.append(random.choice(feature['idx2str']))
-    return ' '.join(text)
+        text.append(random.choice(feature["idx2str"]))
+    return " ".join(text)
 
 
 def generate_timeseries(feature):
     series = []
-    for _ in range(feature.get('max_len', 10)):
-        series.append(str(
-            random.uniform(feature.get('min', 0), feature.get('max', 1))
-        ))
-    return ' '.join(series)
+    for _ in range(feature.get("max_len", 10)):
+        series.append(str(random.uniform(feature.get("min", 0), feature.get("max", 1))))
+    return " ".join(series)
 
 
 def generate_audio(feature):
-    try:
-        import soundfile
-    except ImportError:
-        logger.error(
-            ' soundfile is not installed. '
-            'In order to install all audio feature dependencies run '
-            'pip install ludwig[audio]'
-        )
-        sys.exit(-1)
-
-    destination_folder = feature.get('destination_folder', 'audio_files')
+    destination_folder = feature.get("destination_folder", "audio_files")
     if PREPROCESSING in feature:
-        audio_length = feature[PREPROCESSING].get(
-            'audio_file_length_limit_in_s', 2
-        )
+        audio_length = feature[PREPROCESSING].get("audio_file_length_limit_in_s", 2)
     else:
-        audio_length = feature.get('audio_file_length_limit_in_s', 1)
+        audio_length = feature.get("audio_file_length_limit_in_s", 1)
     sampling_rate = 16000
     num_samples = int(audio_length * sampling_rate)
-    audio = np.sin(np.arange(num_samples) / 100 * 2 * np.pi) * 2 * (
-            np.random.random(num_samples) - 0.5)
-    audio_filename = uuid.uuid4().hex[:10].upper() + '.wav'
+    audio = np.sin(np.arange(num_samples) / 100 * 2 * np.pi) * 2 * (np.random.random(num_samples) - 0.5)
+    audio_tensor = torch.tensor(np.array([audio])).type(torch.float32)
+    audio_filename = uuid.uuid4().hex[:10].upper() + ".wav"
 
     try:
         if not os.path.exists(destination_folder):
             os.makedirs(destination_folder)
 
         audio_dest_path = os.path.join(destination_folder, audio_filename)
-        soundfile.write(audio_dest_path, audio, sampling_rate)
+        torchaudio.save(audio_dest_path, audio_tensor, sampling_rate)
 
-    except IOError as e:
-        raise IOError(
-            'Unable to create a folder for audio or save audio to disk.'
-            '{0}'.format(e))
+    except OSError as e:
+        raise OSError("Unable to create a folder for audio or save audio to disk." "{}".format(e))
 
     return audio_dest_path
 
 
 def generate_image(feature):
     try:
-        from skimage.io import imsave
+        from torchvision.io import write_png
     except ImportError:
         logger.error(
-            ' scikit-image is not installed. '
-            'In order to install all image feature dependencies run '
-            'pip install ludwig[image]'
+            " torchvision is not installed. "
+            "In order to install all image feature dependencies run "
+            "pip install ludwig[image]"
         )
         sys.exit(-1)
 
     # Read num_channels, width, height
-    destination_folder = feature.get('destination_folder', 'image_files')
+    destination_folder = feature.get("destination_folder", "image_files")
     if PREPROCESSING in feature:
-        height = feature[PREPROCESSING].get('height', 28)
-        width = feature[PREPROCESSING].get('width', 28)
-        num_channels = feature[PREPROCESSING].get('num_channels', 1)
+        height = feature[PREPROCESSING].get("height", 28)
+        width = feature[PREPROCESSING].get("width", 28)
+        num_channels = feature[PREPROCESSING].get("num_channels", 1)
     else:
-        height = feature.get('height', 28)
-        width = feature.get('width', 28)
-        num_channels = feature.get('num_channels', 1)
+        height = feature.get("height", 28)
+        width = feature.get("width", 28)
+        num_channels = feature.get("num_channels", 1)
 
     if width <= 0 or height <= 0 or num_channels < 1:
-        raise ValueError('Invalid arguments for generating images')
+        raise ValueError("Invalid arguments for generating images")
 
     # Create a Random Image
-    if num_channels == 1:
-        img = np.random.rand(width, height) * 255
-    else:
-        img = np.random.rand(width, height, num_channels) * 255.0
+    img = torch.randint(0, 255, (num_channels, width, height), dtype=torch.uint8)
 
     # Generate a unique random filename
-    image_filename = uuid.uuid4().hex[:10].upper() + '.jpg'
+    image_filename = uuid.uuid4().hex[:10].upper() + ".png"
 
     # Save the image to disk either in a specified location/new folder
     try:
@@ -325,31 +314,26 @@ def generate_image(feature):
             os.makedirs(destination_folder)
 
         image_dest_path = os.path.join(destination_folder, image_filename)
-        imsave(image_dest_path, img.astype('uint8'))
+        # save_image(torch.from_numpy(img.astype("uint8")), image_dest_path)
+        write_png(img, image_dest_path)
 
-    except IOError as e:
-        raise IOError(
-            'Unable to create a folder for images/save image to disk.'
-            '{0}'.format(e))
+    except OSError as e:
+        raise OSError("Unable to create a folder for images/save image to disk." "{}".format(e))
 
     return image_dest_path
 
 
 def generate_datetime(feature):
     """picking a format among different types.
+
     If no format is specified, the first one is used.
     """
-    if 'datetime_format' in feature:
-        datetime_generation_format = DATETIME_FORMATS[
-            feature['datetime_format']
-        ]
-    elif ('preprocessing' in feature and
-          'datetime_format' in feature['preprocessing']):
-        datetime_generation_format = DATETIME_FORMATS[
-            feature['preprocessing']['datetime_format']
-        ]
+    if "datetime_format" in feature:
+        datetime_generation_format = DATETIME_FORMATS[feature["datetime_format"]]
+    elif "preprocessing" in feature and "datetime_format" in feature["preprocessing"]:
+        datetime_generation_format = DATETIME_FORMATS[feature["preprocessing"]["datetime_format"]]
     else:
-        datetime_generation_format = next(iter(DATETIME_FORMATS))
+        datetime_generation_format = DATETIME_FORMATS[next(iter(DATETIME_FORMATS))]
 
     y = random.randint(1, 99)
     Y = random.randint(1, 9999)
@@ -365,12 +349,12 @@ def generate_datetime(feature):
 def generate_h3(feature):
     resolution = random.randint(0, 15)  # valid values [0, 15]
     h3_components = {
-        'mode': 1,  # we can avoid testing other modes
-        'edge': 0,  # only used in other modes
-        'resolution': resolution,
-        'base_cell': random.randint(0, 121),  # valid values [0, 121]
+        "mode": 1,  # we can avoid testing other modes
+        "edge": 0,  # only used in other modes
+        "resolution": resolution,
+        "base_cell": random.randint(0, 121),  # valid values [0, 121]
         # valid values [0, 7]
-        'cells': [random.randint(0, 7) for _ in range(resolution)]
+        "cells": [random.randint(0, 7) for _ in range(resolution)],
     }
 
     return components_to_h3(h3_components)
@@ -378,27 +362,23 @@ def generate_h3(feature):
 
 def generate_vector(feature):
     # Space delimited string with floating point numbers
-    return ' '.join(
-        [str(100 * random.random())
-         for _ in range(feature.get('vector_size', 10))]
-    )
+    return " ".join([str(100 * random.random()) for _ in range(feature.get("vector_size", 10))])
 
 
 generators_registry = {
     BINARY: generate_binary,
-    NUMERICAL: generate_numerical,
+    NUMBER: generate_number,
     CATEGORY: generate_category,
     SET: generate_set,
     BAG: generate_bag,
     SEQUENCE: generate_sequence,
-    TEXT: generate_sequence,
+    TEXT: generate_text,
     TIMESERIES: generate_timeseries,
     IMAGE: generate_image,
     AUDIO: generate_audio,
     H3: generate_h3,
     DATE: generate_datetime,
-    VECTOR: generate_vector
-
+    VECTOR: generate_vector,
 }
 
 category_cycle = 0
@@ -406,9 +386,9 @@ category_cycle = 0
 
 def cycle_category(feature):
     global category_cycle
-    if category_cycle >= len(feature['idx2str']):
+    if category_cycle >= len(feature["idx2str"]):
         category_cycle = 0
-    category = feature['idx2str'][category_cycle]
+    category = feature["idx2str"][category_cycle]
     category_cycle += 1
     return category
 
@@ -426,18 +406,11 @@ def cycle_binary(feature):
         return False
 
 
-cyclers_registry = {
-    'category': cycle_category,
-    'binary': cycle_binary
-}
+cyclers_registry = {"category": cycle_category, "binary": cycle_binary}
 
 
-def cli_synthesize_dataset(
-        dataset_size: int,
-        features: List[dict],
-        output_path: str
-) -> None:
-    """Symthesizes a dataset for testing purposes
+def cli_synthesize_dataset(dataset_size: int, features: List[dict], output_path: str, **kwargs) -> None:
+    """Symthesizes a dataset for testing purposes.
 
     :param dataset_size: (int) size of the dataset
     :param features: (List[dict]) list of features to generate in YAML format.
@@ -453,8 +426,8 @@ def cli_synthesize_dataset(
         {name: text_2, type: text, vocab_size: 20, max_len: 20},
         {name: category_1, type: category, vocab_size: 10},
         {name: category_2, type: category, vocab_size: 15},
-        {name: numerical_1, type: numerical},
-        {name: numerical_2, type: numerical},
+        {name: number_1, type: number},
+        {name: number_2, type: number},
         {name: binary_1, type: binary},
         {name: binary_2, type: binary},
         {name: set_1, type: set, vocab_size: 20, max_len: 20},
@@ -475,8 +448,7 @@ def cli_synthesize_dataset(
     """
     if dataset_size is None or features is None or output_path is None:
         raise ValueError(
-            "Missing one or more required parameters: '--daset_size', "
-            "'--features' or '--output_path'"
+            "Missing one or more required parameters: '--dataset_size', " "'--features' or '--output_path'"
         )
     dataset = build_synthetic_dataset(dataset_size, features)
     save_csv(output_path, dataset)
@@ -484,33 +456,22 @@ def cli_synthesize_dataset(
 
 def cli(sys_argv):
     parser = argparse.ArgumentParser(
-        description='This script generates a synthetic dataset.',
-        prog='ludwig synthesize_dataset',
-        usage='%(prog)s [options]'
+        description="This script generates a synthetic dataset.",
+        prog="ludwig synthesize_dataset",
+        usage="%(prog)s [options]",
     )
+    parser.add_argument("-od", "--output_path", type=str, help="output CSV file path")
+    parser.add_argument("-d", "--dataset_size", help="size of the dataset", type=int, default=100)
     parser.add_argument(
-        '-od',
-        '--output_path',
-        type=str,
-        help='output CSV file path'
-    )
-    parser.add_argument(
-        '-d',
-        '--dataset_size',
-        help='size of the dataset',
-        type=int,
-        default=100
-    )
-    parser.add_argument(
-        '-f',
-        '--features',
-        default='[\
+        "-f",
+        "--features",
+        default="[\
           {name: text_1, type: text, vocab_size: 20, max_len: 20}, \
           {name: text_2, type: text, vocab_size: 20, max_len: 20}, \
           {name: category_1, type: category, vocab_size: 10}, \
           {name: category_2, type: category, vocab_size: 15}, \
-          {name: numerical_1, type: numerical}, \
-          {name: numerical_2, type: numerical}, \
+          {name: number_1, type: number}, \
+          {name: number_2, type: number}, \
           {name: binary_1, type: binary}, \
           {name: binary_2, type: binary}, \
           {name: set_1, type: set, vocab_size: 20, max_len: 20}, \
@@ -527,15 +488,20 @@ def cli(sys_argv):
           {name: h3_2, type: h3}, \
           {name: vector_1, type: vector}, \
           {name: vector_2, type: vector}, \
-        ]',
+        ]",
         type=yaml.safe_load,
-        help='list of features to generate in YAML format. '
-             'Provide a list containing one dictionary for each feature, '
-             'each dictionary must include a name, a type '
-             'and can include some generation parameters depending on the type'
+        help="list of features to generate in YAML format. "
+        "Provide a list containing one dictionary for each feature, "
+        "each dictionary must include a name, a type "
+        "and can include some generation parameters depending on the type",
     )
+    add_contrib_callback_args(parser)
     args = parser.parse_args(sys_argv)
 
+    args.callbacks = args.callbacks or []
+    for callback in args.callbacks:
+        callback.on_cmdline("synthesize_dataset", *sys_argv)
+
     # No log level parameter this is placeholder if we add at later date
     # args.logging_level = logging_level_registry[args.logging_level]
     # logging.getLogger('ludwig').setLevel(
@@ -544,14 +510,10 @@ def cli(sys_argv):
     # global logger
     # logger = logging.getLogger('ludwig.data.dataset_synthesizer')
 
-
-    if is_on_master():
-        print_ludwig('Synthesize Dataset', LUDWIG_VERSION)
+    print_ludwig("Synthesize Dataset", LUDWIG_VERSION)
 
     cli_synthesize_dataset(**vars(args))
 
 
-if __name__ == '__main__':
-    contrib_import()
-    contrib_command("synthesize_dataset", *sys.argv)
+if __name__ == "__main__":
     cli(sys.argv[1:])
