@@ -19,6 +19,7 @@ from typing import Any, Dict, List, Tuple
 
 import numpy as np
 import pandas as pd
+import torch
 
 from ludwig.backend import Backend, LOCAL_BACKEND
 from ludwig.constants import (
@@ -1132,6 +1133,7 @@ def build_dataset(
             if reshape is not None:
                 proc_cols[proc_column] = backend.df_engine.map_objects(proc_cols[proc_column], lambda x: x.reshape(-1))
 
+    # Implements an outer join of proc_cols
     dataset = backend.df_engine.df_like(dataset_df, proc_cols)
 
     # At this point, there should be no missing values left in the dataframe, unless
@@ -1139,6 +1141,15 @@ def build_dataset(
     # rows.
     dataset = dataset.dropna()
 
+    # NaNs introduced by outer join change dtype of dataset cols (upcast to float64), so we need to cast them back.
+    col_name_to_dtype = {}
+    for col_name, col in proc_cols.items():
+        # if col is a list of list-like objects, we assume the internal dtype of each col[i] remains unchanged.
+        if type(col) == list and type(col[0]) in {list, np.ndarray, torch.Tensor}:
+            continue
+        col_name_to_dtype[col_name] = col.dtype
+    dataset = dataset.astype(col_name_to_dtype)
+
     return dataset, metadata
 
 
