@@ -23,6 +23,7 @@ from ludwig.constants import TEXT
 from ludwig.encoders.base import Encoder
 from ludwig.encoders.registry import register_encoder
 from ludwig.modules.reduction_modules import SequenceReducer
+from ludwig.utils.pytorch_utils import freeze_parameters
 
 logger = logging.getLogger(__name__)
 
@@ -113,6 +114,8 @@ class ALBERTEncoder(Encoder):
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
         self.max_sequence_length = max_sequence_length
 
@@ -237,6 +240,8 @@ class MT5Encoder(Encoder):
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
         self.max_sequence_length = max_sequence_length
 
@@ -331,6 +336,8 @@ class XLMRoBERTaEncoder(Encoder):
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
         self.max_sequence_length = max_sequence_length
 
@@ -448,8 +455,12 @@ class BERTEncoder(Encoder):
         self.reduce_output = reduce_output
         if not self.reduce_output == "cls_pooled":
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
+
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
+
         self.transformer.resize_token_embeddings(vocab_size)
         self.max_sequence_length = max_sequence_length
 
@@ -699,6 +710,8 @@ class GPTEncoder(Encoder):
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
         self.max_sequence_length = max_sequence_length
 
@@ -799,6 +812,8 @@ class GPT2Encoder(Encoder):
 
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.max_sequence_length = max_sequence_length
         self.reduce_output = reduce_output
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
@@ -880,6 +895,8 @@ class RoBERTaEncoder(Encoder):
             self.transformer = RobertaModel(config)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.reduce_output = reduce_output
         if not self.reduce_output == "cls_pooled":
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
@@ -1012,6 +1029,8 @@ class TransformerXLEncoder(Encoder):
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.max_sequence_length = max_sequence_length
 
     def forward(self, inputs: torch.Tensor, mask: torch.Tensor = None) -> Dict[str, torch.Tensor]:
@@ -1136,6 +1155,8 @@ class XLNetEncoder(Encoder):
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
 
     def forward(self, inputs: torch.Tensor, mask: torch.Tensor = None) -> Dict[str, torch.Tensor]:
@@ -1235,6 +1256,9 @@ class DistilBERTEncoder(Encoder):
 
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
+
         self.reduce_output = reduce_output
         self.max_sequence_length = max_sequence_length
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
@@ -1336,6 +1360,8 @@ class CTRLEncoder(Encoder):
         self.max_sequence_length = max_sequence_length
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.reduce_output = reduce_output
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         self.transformer.resize_token_embeddings(self.vocab_size)
@@ -1442,6 +1468,8 @@ class CamemBERTEncoder(Encoder):
 
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.reduce_output = reduce_output
         if not self.reduce_output == "cls_pooled":
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
@@ -1555,6 +1583,8 @@ class T5Encoder(Encoder):
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
 
     def forward(self, inputs: torch.Tensor, mask: Optional[torch.Tensor] = None) -> Dict[str, torch.Tensor]:
@@ -1688,6 +1718,8 @@ class FlauBERTEncoder(Encoder):
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
 
     def forward(self, inputs: torch.Tensor, mask: Optional[torch.Tensor] = None) -> Dict[str, torch.Tensor]:
@@ -1799,6 +1831,8 @@ class ELECTRAEncoder(Encoder):
         self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
 
     def forward(self, inputs: torch.Tensor, mask: Optional[torch.Tensor] = None) -> Dict[str, torch.Tensor]:
@@ -1881,6 +1915,8 @@ class LongformerEncoder(Encoder):
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(num_tokens)
         self.max_sequence_length = max_sequence_length
 
@@ -1955,6 +1991,8 @@ class AutoTransformerEncoder(Encoder):
             self.reduce_sequence = SequenceReducer(reduce_mode=reduce_output)
         if trainable:
             self.transformer.train()
+        else:
+            freeze_parameters(self.transformer)
         self.transformer.resize_token_embeddings(vocab_size)
         self.vocab_size = vocab_size
         self.max_sequence_length = max_sequence_length
