@@ -520,7 +520,18 @@ def run_api_experiment(input_features, output_features, data_csv):
         shutil.rmtree(output_dir, ignore_errors=True)
 
 
-def create_data_set_to_use(data_format, raw_data):
+def read_csv_with_nan(path, nan_percent=0.0):
+    """Converts `nan_percent` of samples in each row of the CSV at `path` to NaNs."""
+    df = pd.read_csv(path)
+    if nan_percent > 0:
+        num_rows = len(df)
+        for col in df.columns:
+            for row in random.sample(range(num_rows), int(round(nan_percent * num_rows))):
+                df[col].iloc[row] = np.nan
+    return df
+
+
+def create_data_set_to_use(data_format, raw_data, nan_percent=0.0):
     # helper function for generating training and test data with specified format
     # handles all data formats except for hdf5
     # assumes raw_data is a csv dataset generated by
@@ -539,56 +550,58 @@ def create_data_set_to_use(data_format, raw_data):
     dataset_to_use = None
 
     if data_format == "csv":
+        # Replace the original CSV with a CSV with NaNs
         dataset_to_use = raw_data
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_csv(dataset_to_use, index=False)
 
     elif data_format in {"df", "dict"}:
-        dataset_to_use = pd.read_csv(raw_data)
+        dataset_to_use = read_csv_with_nan(raw_data, nan_percent=nan_percent)
         if data_format == "dict":
             dataset_to_use = dataset_to_use.to_dict(orient="list")
 
     elif data_format == "excel":
         dataset_to_use = replace_file_extension(raw_data, "xlsx")
-        pd.read_csv(raw_data).to_excel(dataset_to_use, index=False)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_excel(dataset_to_use, index=False)
 
     elif data_format == "excel_xls":
         dataset_to_use = replace_file_extension(raw_data, "xls")
-        pd.read_csv(raw_data).to_excel(dataset_to_use, index=False)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_excel(dataset_to_use, index=False)
 
     elif data_format == "feather":
         dataset_to_use = replace_file_extension(raw_data, "feather")
-        pd.read_csv(raw_data).to_feather(dataset_to_use)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_feather(dataset_to_use)
 
     elif data_format == "fwf":
         dataset_to_use = replace_file_extension(raw_data, "fwf")
-        pd.read_csv(raw_data).to_fwf(dataset_to_use)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_fwf(dataset_to_use)
 
     elif data_format == "html":
         dataset_to_use = replace_file_extension(raw_data, "html")
-        pd.read_csv(raw_data).to_html(dataset_to_use, index=False)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_html(dataset_to_use, index=False)
 
     elif data_format == "json":
         dataset_to_use = replace_file_extension(raw_data, "json")
-        pd.read_csv(raw_data).to_json(dataset_to_use, orient="records")
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_json(dataset_to_use, orient="records")
 
     elif data_format == "jsonl":
         dataset_to_use = replace_file_extension(raw_data, "jsonl")
-        pd.read_csv(raw_data).to_json(dataset_to_use, orient="records", lines=True)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_json(dataset_to_use, orient="records", lines=True)
 
     elif data_format == "parquet":
         dataset_to_use = replace_file_extension(raw_data, "parquet")
-        pd.read_csv(raw_data).to_parquet(dataset_to_use, index=False)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_parquet(dataset_to_use, index=False)
 
     elif data_format == "pickle":
         dataset_to_use = replace_file_extension(raw_data, "pickle")
-        pd.read_csv(raw_data).to_pickle(dataset_to_use)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_pickle(dataset_to_use)
 
     elif data_format == "stata":
         dataset_to_use = replace_file_extension(raw_data, "stata")
-        pd.read_csv(raw_data).to_stata(dataset_to_use)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_stata(dataset_to_use)
 
     elif data_format == "tsv":
         dataset_to_use = replace_file_extension(raw_data, "tsv")
-        pd.read_csv(raw_data).to_csv(dataset_to_use, sep="\t", index=False)
+        read_csv_with_nan(raw_data, nan_percent=nan_percent).to_csv(dataset_to_use, sep="\t", index=False)
 
     else:
         ValueError(f"'{data_format}' is an unrecognized data format")
@@ -660,7 +673,9 @@ def train_with_backend(
                     assert k1 == k2
                     for (name1, metric1), (name2, metric2) in zip(v1.items(), v2.items()):
                         assert name1 == name2
-                        assert np.isclose(metric1, metric2, rtol=1e-04), f"metric {name1}: {metric1} != {metric2}"
+                        assert np.isclose(
+                            metric1, metric2, rtol=1e-04, atol=1e-5
+                        ), f"metric {name1}: {metric1} != {metric2}"
 
         return model
     finally:
